@@ -7,11 +7,15 @@
  *
  ***************************************************************************/
 
-#include <zorp/process.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#include <zorpll/process.h>
  
 #ifndef G_OS_WIN32
 
-#include <zorp/cap.h>
+#include <zorpll/cap.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -32,12 +36,23 @@
 #include <pwd.h>
 #include <grp.h>
 
-#include <zorp/misc.h>
+#include <chrono>
+#include <exception>
+#include <string>
+#include <thread>
+
+#include <zorpll/misc.h>
 
 #if HAVE_SYS_PRCTL_H
 #  include <sys/prctl.h>
 #endif
 
+#if HAVE_SYSTEMD == 1
+#include <systemd/sd-daemon.h>
+#include <zorpll/thread.h>
+#include <zorpll/log.h>
+#endif
+
 /**
  * @file
  *
@@ -182,6 +197,7 @@ static struct
   const gchar *pidfile_dir;
   const gchar *cwd;
   const gchar *caps;
+  gboolean nocaps;
   gchar *argv_orig;
   gboolean core;
 } process_opts =
@@ -451,6 +467,16 @@ z_process_get_check_enable(void)
   return FALSE;
 }
 
+/**
+ * Returns with the current processing mode.
+ *
+ * @returns current processing mode
+ **/
+ZProcessMode
+z_process_get_mode()
+{
+  return process_opts.mode;
+}
 
 /**
  * Send a message to the client using stderr as long as it's available and using
@@ -495,7 +521,7 @@ z_process_message(const gchar *fmt, ...)
 static void
 z_process_detach_tty(void)
 {
-  if (process_opts.mode != Z_PM_FOREGROUND)
+  if (process_opts.mode != Z_PM_FOREGROUND && process_opts.mode != Z_PM_SYSTEMD_NOTIFY)
     {
       /* detach ourselves from the tty when not staying in the foreground */
       if (isatty(STDIN_FILENO))
@@ -532,7 +558,7 @@ z_process_detach_stdio(void)
 {
   gint devnull_fd;
 
-  if (process_opts.mode != Z_PM_FOREGROUND && stderr_present)
+  if (process_opts.mode != Z_PM_FOREGROUND && process_opts.mode != Z_PM_SYSTEMD_NOTIFY && stderr_present)
     {
       devnull_fd = open("/dev/null", O_RDONLY);
       if (devnull_fd >= 0)
@@ -730,7 +756,7 @@ z_process_change_user(void)
   gid_t gid = -1;
   
 #if HAVE_PRCTL && HAVE_PR_SET_KEEPCAPS
-  if (process_opts.caps)
+  if (process_opts.caps && !process_opts.nocaps)
     prctl(PR_SET_KEEPCAPS, 1, 0, 0, 0);
 #endif
 
@@ -787,7 +813,7 @@ static gboolean
 z_process_change_caps(void)
 {
 #if ZORPLIB_ENABLE_CAPS
-  if (process_opts.caps)
+  if (process_opts.caps && !process_opts.nocaps)
     {
       cap_t cap = cap_from_text(process_opts.caps);
 
@@ -819,7 +845,7 @@ static void
 z_process_change_dir(void)
 {
   const gchar *cwd = NULL;
-  
+
   if (process_opts.mode != Z_PM_FOREGROUND)
     {
       if (process_opts.cwd)
@@ -1235,6 +1261,22 @@ z_process_perform_supervise(void)
   exit(0);
 }
 
+#if HAVE_SYSTEMD == 1
+static gpointer
+z_systemd_notify(gpointer user_data)
+{
+  const std::chrono::microseconds watchdog_delay{GPOINTER_TO_INT(user_data)};
+
+  while (true)
+    {
+      std::this_thread::sleep_for(watchdog_delay);
+      sd_notify(0, "WATCHDOG=1");
+    }
+
+  return nullptr;
+}
+#endif
+
 /**
  * Start the process as directed by the options set by various
  * z_process_set_*() functions.
@@ -1322,14 +1364,16 @@ z_process_start(void)
     {
       process_kind = Z_PK_DAEMON;
     }
+#if HAVE_SYSTEMD == 1
+  else if (process_opts.mode == Z_PM_SYSTEMD_NOTIFY)
+    {
+      process_kind = Z_PK_DAEMON;
+    }
+#endif
   else
     {
       g_assert_not_reached();
     }
-    
-  /* daemon process, we should return to the caller to perform work */
-  
-  setsid();
   
   /* NOTE: we need to signal the parent in case of errors from this point. 
    * This is accomplished by writing the appropriate exit code to
@@ -1343,7 +1387,9 @@ z_process_start(void)
       z_process_startup_failed(1, TRUE);
     }
   z_process_enable_core();
-  z_process_change_dir();
+
+  if (process_opts.mode != Z_PM_SYSTEMD_NOTIFY)
+    z_process_change_dir();
 }
 
 
@@ -1375,7 +1421,41 @@ z_process_startup_failed(guint ret_num, gboolean may_exit)
 void
 z_process_startup_ok(void)
 {
-  z_process_write_pidfile(getpid());
+#if HAVE_SYSTEMD == 1
+  if (process_opts.mode == Z_PM_SYSTEMD_NOTIFY)
+    {
+      const char *watchdog_usec_env_str = getenv("WATCHDOG_USEC");
+      if (!watchdog_usec_env_str)
+        {
+          z_log(nullptr, CORE_ERROR, 0,
+                "The systemd-notify process-mode requires the WATCHDOG_USEC environment variable to be set");
+          exit(1);
+        }
+
+      int watchdog_delay_usec = 0;
+      try
+        {
+          std::size_t pos = 0;
+          watchdog_delay_usec = std::stoi(watchdog_usec_env_str, &pos) / 2; // sleep period need to be half of this value
+          if (pos != std::string{watchdog_usec_env_str}.length())
+            throw std::invalid_argument{"Value is not a number"};
+        }
+      catch (const std::exception &e)
+        {
+          z_log(nullptr, CORE_ERROR, 0, "WATCHDOG_USEC has invalid value; error='%s'", e.what());
+          exit(1);
+        }
+
+      z_thread_new("systemd-notify", z_systemd_notify, GINT_TO_POINTER(watchdog_delay_usec));
+
+      sd_notify(0, "READY=1");
+    }
+#endif
+
+  if (process_opts.mode != Z_PM_SYSTEMD_NOTIFY)
+    {
+      z_process_write_pidfile(getpid());
+    }
   
   z_process_send_result(0);
   z_process_detach_stdio();
@@ -1389,7 +1469,10 @@ z_process_startup_ok(void)
 void
 z_process_finish(void)
 {
-  z_process_remove_pidfile();
+  if (process_opts.mode != Z_PM_SYSTEMD_NOTIFY)
+    {
+      z_process_remove_pidfile();
+    }
 }
 
 /**
@@ -1399,11 +1482,14 @@ z_process_finish(void)
 void
 z_process_finish_prepare(void)
 {
-  z_process_remove_pidfile();
+  if (process_opts.mode != Z_PM_SYSTEMD_NOTIFY)
+    {
+      z_process_remove_pidfile();
+    }
 }
 
 static gboolean
-z_process_process_mode_arg(const gchar *option_name G_GNUC_UNUSED, const gchar *value, gpointer data G_GNUC_UNUSED, GError **error)
+z_process_process_mode_arg(const gchar */* option_name */, const gchar *value, gpointer /* data */, GError **error)
 {
   if (strcmp(value, "foreground") == 0)
     {
@@ -1417,6 +1503,12 @@ z_process_process_mode_arg(const gchar *option_name G_GNUC_UNUSED, const gchar *
     {
       process_opts.mode = Z_PM_SAFE_BACKGROUND;
     }
+#if HAVE_SYSTEMD == 1
+  else if (strcmp(value, "systemd-notify") == 0)
+    {
+      process_opts.mode = Z_PM_SYSTEMD_NOTIFY;
+    }
+#endif
   else
     {
       g_set_error(error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, "Error parsing process-mode argument");
@@ -1428,14 +1520,19 @@ z_process_process_mode_arg(const gchar *option_name G_GNUC_UNUSED, const gchar *
 static GOptionEntry z_process_option_entries[] =
 {
   { "foreground",   'F', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE,     &process_opts.mode,              "Do not go into the background after initialization", NULL },
-  { "process-mode",   0,                     0, G_OPTION_ARG_CALLBACK, reinterpret_cast<void *>(z_process_process_mode_arg) ,     "Set process running mode", "<foreground|background|safe-background>" },
+  { "process-mode",   0,                     0, G_OPTION_ARG_CALLBACK, reinterpret_cast<void *>(z_process_process_mode_arg) ,     "Set process running mode",
+#if HAVE_SYSTEMD == 1
+    "<foreground|background|safe-background|systemd-notify>" },
+#else
+    "<foreground|background|safe-background>" },
+#endif
   { "user",         'u',                     0, G_OPTION_ARG_STRING,   &process_opts.user,              "Set the user to run as", "<user>" },
   { "uid",            0,  G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING,   &process_opts.user,              NULL, NULL },
   { "group",        'g',                     0, G_OPTION_ARG_STRING,   &process_opts.group,             "Set the group to run as", "<group>" },
   { "gid",            0,  G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING,   &process_opts.group,             NULL, NULL },
   { "chroot",       'R',                     0, G_OPTION_ARG_STRING,   &process_opts.chroot_dir,        "Chroot to this directory", "<dir>" },
   { "caps",         'C',                     0, G_OPTION_ARG_STRING,   &process_opts.caps,              "Set default capability set", "<capspec>" },
-  { "no-caps",      'N', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE,     &process_opts.caps,              "Disable managing Linux capabilities", NULL },
+  { "no-caps",      'N',                     0, G_OPTION_ARG_NONE,     &process_opts.nocaps,            "Disable managing Linux capabilities", NULL },
   { "pidfile",      'P',                     0, G_OPTION_ARG_STRING,   &process_opts.pidfile,           "Set path to pid file", "<pidfile>" },
   { "enable-core",    0,                     0, G_OPTION_ARG_NONE,     &process_opts.core,              "Enable dumping core files", NULL },
   { "fd-limit-min",       0,                  0, G_OPTION_ARG_INT,      &process_opts.fd_limit_min,      "The minimum required number of fds", NULL },
