@@ -7,11 +7,11 @@
  *
  ***************************************************************************/
 
-#include <zorp/stream.h>
-#include <zorp/log.h>
-#include <zorp/error.h>
-#include <zorp/packetbuf.h>
-#include <zorp/io.h>
+#include <zorpll/stream.h>
+#include <zorpll/log.h>
+#include <zorpll/error.h>
+#include <zorpll/packetbuf.h>
+#include <zorpll/io.h>
 
 #include <string.h>
 #include <sys/types.h>
@@ -206,8 +206,8 @@ z_stream_source_check(GSource *s)
 
 static gboolean
 z_stream_source_dispatch(GSource     *s,
-                         GSourceFunc callback G_GNUC_UNUSED,
-                         gpointer    user_data G_GNUC_UNUSED)
+                         GSourceFunc /* callback */,
+                         gpointer    /* user_data */)
 {
   ZStreamSource *self = (ZStreamSource *) s;
   gboolean ret = FALSE;
