@@ -7,10 +7,10 @@
  *
  ***************************************************************************/
 
-#include <zorp/streamfd.h>
+#include <zorpll/streamfd.h>
 
-#include <zorp/log.h>
-#include <zorp/error.h>
+#include <zorpll/log.h>
+#include <zorpll/error.h>
 
 #include <string.h>
 #include <sys/types.h>
@@ -18,7 +18,7 @@
 
 #ifdef G_OS_WIN32
 #  include <winsock2.h>
-#  include <zorp/io.h>
+#  include <zorpll/io.h>
 #else
 #  include <fcntl.h>
 #  include <sys/socket.h>
@@ -56,7 +56,7 @@ typedef struct _ZStreamFDExtra
 } ZStreamFDExtra;
 
 static gboolean
-z_stream_fd_watch_prepare(ZStream *s, GSource *src G_GNUC_UNUSED, gint *timeout)
+z_stream_fd_watch_prepare(ZStream *s, GSource */* src */, gint *timeout)
 {
   ZStreamFD *mystream = (ZStreamFD *) s;
   GPollFD *mypollfd = &mystream->pollfd;
@@ -113,7 +113,7 @@ z_stream_fd_watch_prepare(ZStream *s, GSource *src G_GNUC_UNUSED, gint *timeout)
     {
       /*LOG
         This message indicates an internal error during WSAEventSelect. Please report this event
-	to the Balabit QA Team (devel@balabit.com).
+	to the BalaSys Development Team (devel@balasys.hu).
        */
       z_log(NULL, CORE_ERROR, 1, "Failed to set up WinSock event for z_stream_fd; error_code='%d'", z_errno_get());
       z_return(FALSE);
@@ -147,7 +147,7 @@ z_stream_fd_watch_prepare(ZStream *s, GSource *src G_GNUC_UNUSED, gint *timeout)
 }
 
 static gboolean
-z_stream_fd_watch_check(ZStream *s, GSource *src G_GNUC_UNUSED)
+z_stream_fd_watch_check(ZStream *s, GSource */* src */)
 {
   ZStreamFD *mystream = (ZStreamFD *) s;
   GPollFD *mypollfd = &mystream->pollfd;
@@ -246,7 +246,7 @@ z_stream_fd_watch_dispatch(ZStream *s, GSource *src)
         {
           /*LOG
             This message indicates an internal error, read event occurred, but no read
-            callback is set. Please report this event to the Balabit QA Team (devel@balabit.com).
+            callback is set. Please report this event to the BalaSys Development Team (devel@balasys.hu).
            */
           z_log(mystream->super.name, CORE_ERROR, 3, "Internal error, no read callback is set;");
         }
@@ -262,7 +262,7 @@ z_stream_fd_watch_dispatch(ZStream *s, GSource *src)
         {
 	  /*LOG
 	    This message indicates an internal error, write event occurred, but no write
-	    callback is set. Please report this event to the Balabit QA Team (devel@balabit.com).
+	    callback is set. Please report this event to the BalaSys Development Team (devel@balasys.hu).
 	   */
           z_log(mystream->super.name, CORE_ERROR, 3, "Internal error, no write callback is set;");
         }
@@ -278,7 +278,7 @@ z_stream_fd_watch_dispatch(ZStream *s, GSource *src)
         {
 	  /*LOG
 	    This message indicates an internal error, pri-read event occurred, but no pri
-	    callback is set. Please report this event to the Balabit QA Team (devel@balabit.com).
+	    callback is set. Please report this event to the BalaSys Development Team (devel@balasys.hu).
 	   */
           z_log(mystream->super.name, CORE_ERROR, 3, "Internal error, no pri callback is set;");
         }
@@ -683,7 +683,7 @@ z_stream_fd_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
 
           /*LOG
             This message indicates that an internal error occurred, during setting CLOSE_ON_EXEC mode
-            on a stream. Please report this event to the Balabit QA Team (devel@balabit.com).
+            on a stream. Please report this event to the BalaSys Development Team (devel@balasys.hu).
            */
           z_log(NULL, CORE_ERROR, 4, "Internal error, during setting CLOSE_ON_EXEC mode;");
         }
@@ -692,7 +692,7 @@ z_stream_fd_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
           /*LOG
             This message indicates that an internal error occurred, during setting CLOSE_ON_EXEC
             mode on a stream, because the size of the parameter is wrong. Please report this
-            event to the Balabit QA Team (devel@balabit.com).
+            event to the BalaSys Development Team (devel@balasys.hu).
            */
           z_log(NULL, CORE_ERROR, 4, "Internal error, bad parameter is given for setting CLOSE_ON_EXEC mode; size='%d'", vlen);
         }
@@ -720,7 +720,7 @@ z_stream_fd_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
 
           /*LOG
             This message indicates that an internal error, during setting NONBLOCK mode on
-            a stream. Please report this event to the Balabit QA Team (devel@balabit.com).
+            a stream. Please report this event to the BalaSys Development Team (devel@balasys.hu).
            */
           z_log(NULL, CORE_ERROR, 4, "Internal error, during setting NONBLOCK mode;");
         }
@@ -728,7 +728,7 @@ z_stream_fd_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
         /*LOG
           This message indicates that an internal error occurred, during setting NONBLOCK mode
           on a stream, because the size of the parameter is wrong. Please report this event to
-          the Balabit QA Team (devel@balabit.com).
+          the BalaSys Development Team (devel@balasys.hu).
          */
         z_log(NULL, CORE_ERROR, 4, "Internal error, bad parameter is given for setting NONBLOCK mode; size='%d'", vlen);
       break;
@@ -745,7 +745,7 @@ z_stream_fd_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
       /*LOG
         This message indicates that an internal error occurred, during getting NONBLOCK mode status
         on a stream, because the size of the parameter is wrong. Please report this event to
-        the Balabit QA Team (devel@balabit.com).
+        the BalaSys Development Team (devel@balasys.hu).
        */
       z_log(NULL, CORE_ERROR, 4, "Internal error, bad parameter is given for getting the NONBLOCK mode; size='%d'", vlen);
       break;
@@ -758,8 +758,8 @@ z_stream_fd_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
         }
       /*LOG
         This message indicates that an internal error occurred, during getting the FD of a stream,
-        because the size of the parameter is wrong. Please report this event to the Balabit QA
-        Team (devel@balabit.com).
+        because the size of the parameter is wrong. Please report this event to the BalaSys Development
+        Team (devel@balasys.hu).
        */
       z_log(NULL, CORE_ERROR, 4, "Internal error, bad parameter is given for getting the FD; size='%d'", vlen);
       break;
@@ -780,8 +780,8 @@ z_stream_fd_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
         }
       /*LOG
         This message indicates that an internal error occurred, during getting the FD of a stream,
-        because the size of the parameter is wrong. Please report this event to the Balabit QA
-        Team (devel@balabit.com).
+        because the size of the parameter is wrong. Please report this event to the BalaSys Development
+        Team (devel@balasys.hu).
        */
       z_log(NULL, CORE_ERROR, 4, "Internal error, bad parameter is given for getting the broken state; size='%d'", vlen);
 #else
@@ -800,8 +800,8 @@ z_stream_fd_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
       else
         /*LOG
           This message indicates that an internal error occurred, during getting the FD of a stream,
-          because the size of the parameter is wrong. Please report this event to the Balabit QA
-          Team (devel@balabit.com).
+          because the size of the parameter is wrong. Please report this event to the BalaSys Development
+          Team (devel@balasys.hu).
           */
         z_log(NULL, CORE_ERROR, 4, "Internal error, bad parameter is given for getting the KEEPALIVE option; size='%d'", vlen);
       break;
@@ -816,8 +816,8 @@ z_stream_fd_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
       else
         /*LOG
           This message indicates that an internal error occurred, during getting the FD of a stream,
-          because the size of the parameter is wrong. Please report this event to the Balabit QA
-          Team (devel@balabit.com).
+          because the size of the parameter is wrong. Please report this event to the BalaSys Development
+          Team (devel@balasys.hu).
           */
         z_log(NULL, CORE_ERROR, 4, "Internal error, bad parameter is given for setting the KEEPALIVE option; size='%d'", vlen);
       break;
@@ -827,7 +827,7 @@ z_stream_fd_ctrl_method(ZStream *s, guint function, gpointer value, guint vlen)
         z_return(TRUE);
       /*LOG
         This message indicates that an internal error occurred, because an invalid control was called.
-        Please report this event to the Balabit QA Team (devel@balabit.com).
+        Please report this event to the BalaSys Development Team (devel@balasys.hu).
        */
       z_log(NULL, CORE_ERROR, 4, "Internal error, unknown stream ctrl; ctrl='%d'", ZST_CTRL_MSG(function));
       break;
