@@ -1,9 +1,9 @@
 #define BOOST_TEST_MAIN
 #include <boost/test/unit_test.hpp>
 
-#include <zorp/listen.h>
-#include <zorp/connect.h>
-#include <zorp/sockaddr.h>
+#include <zorpll/listen.h>
+#include <zorpll/connect.h>
+#include <zorpll/sockaddr.h>
 
 #include <stdio.h>
 
@@ -19,7 +19,7 @@ test_done(void)
 }
 
 static gboolean
-test_accepted(ZStream *stream G_GNUC_UNUSED, ZSockAddr *client, ZSockAddr *dest, gpointer user_data G_GNUC_UNUSED)
+test_accepted(ZStream * /* stream */, ZSockAddr *client, ZSockAddr *dest, gpointer /* user_data */)
 {
   printf("Connection accepted\n");
   z_sockaddr_unref(client);
@@ -29,24 +29,35 @@ test_accepted(ZStream *stream G_GNUC_UNUSED, ZSockAddr *client, ZSockAddr *dest,
 }
 
 static void
-test_connected(ZStream *fdstream G_GNUC_UNUSED, GError *error G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
+test_connected(ZStream * /* fdstream */, GError * /* error */, gpointer /* user_data */)
 {
   printf("Connected\n");
   test_done();
 }
 
+class TempFile
+{
+public:
+   TempFile() { name = tmpnam(NULL); };
+   ~TempFile() { unlink(name); }
+   char *get() { return name; }
+private:
+   char *name;
+};
+
 BOOST_AUTO_TEST_CASE(test_conns)
 {
   ZSockAddr *a;
   ZListener *l;
   ZConnector *c;
   ZSockAddr *dest;
+  TempFile socket_name;
 
   loop = g_main_loop_new(NULL, TRUE);
 
-  a = z_sockaddr_unix_new("sock");
+  a = z_sockaddr_unix_new(socket_name.get());
   l = z_stream_listener_new("sessionid", a, 0, 255, test_accepted, NULL);
-  BOOST_CHECK(z_listener_start(l));
+  BOOST_REQUIRE(z_listener_start(l));
 
   c = z_stream_connector_new("sessionid", NULL, a, 0, test_connected, NULL, NULL);
   z_connector_start(c, &dest);
